﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Linq.Expressions;
using System.Text;
using System.Threading.Tasks;
using DAL;
using System.Data;
using System.Data.Entity;
using MvcCoffeeShop.Repository.Helpers;
using System.Data.Objects.DataClasses;
using System.Reflection;

namespace MvcCoffeeShop.Repository
{
    public class GenericRepository<TEntity,TDtoEntity> where TEntity : class 

    {
        internal MvcCoffeeShopEntities context;
        internal DbSet<TEntity> dbSet;
        private TDtoEntity tdtoentity;
        private TEntity entity;
        public GenericRepository(MvcCoffeeShopEntities context, TEntity entity, TDtoEntity tdtoentity)
        {
            context = new MvcCoffeeShopEntities();
            this.context = context;
            this.dbSet = context.Set<TEntity>();
            this.tdtoentity = tdtoentity;
            this.entity = entity;
        
        }

        public virtual IEnumerable<TEntity> Get(
            Expression<Func<TEntity, bool>> filter = null,
            Func<IQueryable<TEntity>, IOrderedQueryable<TEntity>> orderBy = null,
            string includeProperties = "")
        {
            IQueryable<TEntity> query = dbSet;

            if (filter != null)
            {
                query = query.Where(filter);
            }

            foreach (var includeProperty in includeProperties.Split
                (new char[] { ',' }, StringSplitOptions.RemoveEmptyEntries))
            {
                query = query.Include(includeProperty);
            }

            if (orderBy != null)
            {
                return orderBy(query).ToList();
            }
            else
            {
                return query.ToList();
            }
        }

        public virtual TDtoEntity GetByID(object id)
        {
           
            var entity= dbSet.Find(id);
            CopyToDTO( entity ,  tdtoentity );
            return tdtoentity;
           
        }

        public virtual void Insert(TDtoEntity tdtoentity)
        {
             
            CopyFromDTO(entity, tdtoentity);            
            dbSet.Add(entity);
        }

        public virtual void Delete(object id)
        {
            TEntity entityToDelete = dbSet.Find(id);
            Delete(entityToDelete);
        }

        public virtual void Delete(TEntity entityToDelete,TDtoEntity obj)
        {

            if (context.Entry(entityToDelete).State == EntityState.Detached)
            {
                dbSet.Attach(entityToDelete);
            }
            dbSet.Remove(entityToDelete);
        }

        public virtual void Update(TEntity entityToUpdate)
        {
            dbSet.Attach(entityToUpdate);
            context.Entry(entityToUpdate).State = EntityState.Modified;
        }

        public virtual void CopyFromDTO(TEntity entity, Object dto)
        {
            Type entityType = entity.GetType();

            Type dtoType = dto.GetType();

            PropertyInfo[] dtoProperties = dtoType.GetProperties();

            foreach (var dtoProperty in dtoProperties)
            {
                PropertyInfo entityProperty = entityType.GetProperty(dtoProperty.Name);
                if (entityProperty != null && dtoProperty.CanWrite)
                {

                    entityProperty.SetValue(entity, dtoProperty.GetValue(dto, null), null);
                }

            }

        }



        public virtual void CopyToDTO(TEntity entity, Object dto)
        {
            Type entityType = entity.GetType();

            Type dtoType = dto.GetType();

            PropertyInfo[] dtoProperties = dtoType.GetProperties();

            foreach (var dtoProperty in dtoProperties)
            {
                PropertyInfo entityProperty = entityType.GetProperty(dtoProperty.Name);
                if (entityProperty != null)
                {

                    dtoProperty.SetValue(dto, entityType.GetProperty(dtoProperty.Name).GetValue(entity, null), null);
                }

            }

        }
    }
}
